@@ -11,7 +11,9 @@
 #include <stdbool.h>
 #endif
 
-__BEGIN_DECLS
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /**
  * Flags for cgroup_delete_cgroup_ext().
@@ -149,6 +151,16 @@ struct cgroup *cgroup_new_cgroup(const char *name);
 struct cgroup_controller *cgroup_add_controller(struct cgroup *cgroup,
 						const char *name);
 
+/**
+ * Attach all mounted controllers to given cgroup. This function just modifies
+ * internal libcgroup structure, not the kernel control group.
+ *
+ * @param cgroup
+ * @return zero or error number
+ */
+int cgroup_add_all_controllers(struct cgroup *cgroup);
+
+
 /**
  * Return appropriate controller from given group.
  * The controller must be added before using cgroup_add_controller() or loaded
@@ -295,7 +307,8 @@ int cgroup_delete_cgroup_ext(struct cgroup *cgroup, int flags);
 int cgroup_get_cgroup(struct cgroup *cgroup);
 
 /**
- * Copy all controllers, parameters and their values. All existing controllers
+ * Copy all controllers, their parameters and values. Group name, permissions
+ * and ownerships are not coppied. All existing controllers
  * in the source group are discarded.
  *
  * @param dst Destination group.
@@ -564,12 +577,20 @@ int cgroup_get_procs(char *name, char *controller, pid_t **pids, int *size);
 int cg_chmod_recursive(struct cgroup *cgroup, mode_t dir_mode,
 	int dirm_change, mode_t file_mode, int filem_change);
 
+/**
+ *  Get the name of the cgroup from a given cgroup
+ *  @param cgroup The cgroup whose name is needed
+ */
+char *cgroup_get_cgroup_name(struct cgroup *cgroup);
+
 /**
  * @}
  * @}
  */
 
 
-__END_DECLS
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
 
 #endif /* _LIBCGROUP_GROUPS_H */
