@@ -18,15 +18,17 @@
 
 #define __TOOLS_COMMON
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 #include "config.h"
 #include <libcgroup.h>
-#include <libcgroup-internal.h>
+#include "../libcgroup-internal.h"
 
-#ifdef CGROUP_DEBUG
-#define cgroup_dbg(x...) printf(x)
-#else
-#define cgroup_dbg(x...) do {} while (0)
-#endif
+#define cgroup_err(x...) cgroup_log(CGROUP_LOG_ERROR, x)
+#define cgroup_warn(x...) cgroup_log(CGROUP_LOG_WARNING, x)
+#define cgroup_info(x...) cgroup_log(CGROUP_LOG_INFO, x)
+#define cgroup_dbg(x...) cgroup_log(CGROUP_LOG_DEBUG, x)
 
 /**
  * Auxiliary specifier of group, used to store parsed command line options.
@@ -123,4 +125,21 @@ int parse_mode(char *string, mode_t *pmode, const char *program_name);
 int parse_uid_gid(char *string, uid_t *uid, gid_t *gid,
 		const char *program_name);
 
+/**
+ * Functions that are defined as STATIC can be placed within the UNIT_TEST
+ * ifdef.  This will allow them to be included in the unit tests while
+ * remaining static in a normal libcgroup build.
+ */
+#ifdef UNIT_TEST
+
+int parse_r_flag(const char * const program_name,
+		 const char * const name_value_str,
+		 struct control_value * const name_value);
+
+#endif /* UNIT_TEST */
+
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
+
 #endif /* TOOLS_COMMON */
