@@ -48,7 +48,7 @@ struct cgroup *cgroup_new_cgroup(const char *name)
 		return NULL;
 
 	init_cgroup(cgroup);
-	strncpy(cgroup->name, name, sizeof(cgroup->name));
+	strncpy(cgroup->name, name, sizeof(cgroup->name) - 1);
 
 	return cgroup;
 }
@@ -82,7 +82,8 @@ struct cgroup_controller *cgroup_add_controller(struct cgroup *cgroup,
 	if (!controller)
 		return NULL;
 
-	strncpy(controller->name, name, sizeof(controller->name));
+	strncpy(controller->name, name, sizeof(controller->name) - 1);
+	controller->cgroup = cgroup;
 	controller->index = 0;
 
 	cgroup->controller[cgroup->index] = controller;
@@ -91,6 +92,56 @@ struct cgroup_controller *cgroup_add_controller(struct cgroup *cgroup,
 	return controller;
 }
 
+int cgroup_add_all_controllers(struct cgroup *cgroup)
+{
+	int ret;
+	void *handle;
+	struct controller_data info;
+	struct cgroup_controller *cgc;
+
+	/* go through the controller list */
+	ret = cgroup_get_all_controller_begin(&handle, &info);
+	if ((ret != 0) && (ret != ECGEOF)) {
+		fprintf(stderr, "cannot read controller data: %s\n",
+			cgroup_strerror(ret));
+		return ret;
+	}
+
+	while (ret == 0) {
+		if (info.hierarchy == 0) {
+			/* the controller is not attached to any hierarchy
+			   skip it */
+			goto next;
+		}
+
+		/* add mounted controller to cgroup structure */
+		cgc = cgroup_add_controller(cgroup, info.name);
+		if (!cgc) {
+			ret = ECGINVAL;
+			fprintf(stderr, "controller %s can't be add\n",
+				info.name);
+		}
+
+next:
+		ret = cgroup_get_all_controller_next(&handle, &info);
+		if (ret && ret != ECGEOF)
+			goto end;
+	}
+
+end:
+	cgroup_get_all_controller_end(&handle);
+
+	if (ret == ECGEOF)
+		ret = 0;
+
+	if (ret)
+		fprintf(stderr,
+			"cgroup_get_controller_begin/next failed (%s)\n",
+			cgroup_strerror(ret));
+
+	return ret;
+}
+
 void cgroup_free_controllers(struct cgroup *cgroup)
 {
 	int i, j;
@@ -99,8 +150,12 @@ void cgroup_free_controllers(struct cgroup *cgroup)
 		return;
 
 	for (i = 0; i < cgroup->index; i++) {
-		for (j = 0; j < cgroup->controller[i]->index; j++)
+		for (j = 0; j < cgroup->controller[i]->index; j++) {
+			if (cgroup->controller[i]->values[j]->multiline_value)
+				free(cgroup->controller[i]->values[j]->multiline_value);
+
 			free(cgroup->controller[i]->values[j]);
+		}
 		cgroup->controller[i]->index = 0;
 		free(cgroup->controller[i]);
 	}
@@ -131,10 +186,10 @@ int cgroup_add_value_string(struct cgroup_controller *controller,
 	if (!controller)
 		return ECGINVAL;
 
-	if (controller->index >= CG_VALUE_MAX)
+	if (controller->index >= CG_NV_MAX)
 		return ECGMAXVALUESEXCEEDED;
 
-	for (i = 0; i < controller->index && i < CG_VALUE_MAX; i++) {
+	for (i = 0; i < controller->index && i < CG_NV_MAX; i++) {
 		if (!strcmp(controller->values[i]->name, name))
 			return ECGVALUEEXISTS;
 	}
@@ -145,7 +200,21 @@ int cgroup_add_value_string(struct cgroup_controller *controller,
 		return ECGCONTROLLERCREATEFAILED;
 
 	strncpy(cntl_value->name, name, sizeof(cntl_value->name));
-	strncpy(cntl_value->value, value, sizeof(cntl_value->value));
+	cntl_value->name[sizeof(cntl_value->name)-1] = '\0';
+
+	if (value) {
+		if (strlen(value) >= sizeof(cntl_value->value)) {
+			fprintf(stderr, "value exceeds the maximum of %ld characters\n",
+				sizeof(cntl_value->value) - 1);
+			free(cntl_value);
+			return ECGCONFIGPARSEFAIL;
+		}
+
+		strncpy(cntl_value->value, value, sizeof(cntl_value->value));
+		cntl_value->value[sizeof(cntl_value->value)-1] = '\0';
+		cntl_value->dirty = true;
+	}
+
 	controller->values[controller->index] = cntl_value;
 	controller->index++;
 
@@ -355,6 +424,8 @@ int cgroup_set_value_string(struct cgroup_controller *controller,
 		struct control_value *val = controller->values[i];
 		if (!strcmp(val->name, name)) {
 			strncpy(val->value, value, CG_VALUE_MAX);
+			val->value[sizeof(val->value)-1] = '\0';
+			val->dirty = true;
 			return 0;
 		}
 	}
@@ -404,6 +475,7 @@ int cgroup_set_value_int64(struct cgroup_controller *controller,
 			if (ret >= sizeof(val->value) || ret < 0)
 				return ECGINVAL;
 
+			val->dirty = true;
 			return 0;
 		}
 	}
@@ -452,6 +524,7 @@ int cgroup_set_value_uint64(struct cgroup_controller *controller,
 			if (ret >= sizeof(val->value) || ret < 0)
 				return ECGINVAL;
 
+			val->dirty = true;
 			return 0;
 		}
 	}
@@ -511,6 +584,7 @@ int cgroup_set_value_bool(struct cgroup_controller *controller,
 			if (ret >= sizeof(val->value) || ret < 0)
 				return ECGINVAL;
 
+			val->dirty = true;
 			return 0;
 
 		}
@@ -547,15 +621,20 @@ struct cgroup *create_cgroup_from_name_value_pairs(const char *name,
 			goto scgroup_err;
 		}
 
-		strncpy(con, name_value[i].name, FILENAME_MAX);
+		strncpy(con, name_value[i].name, FILENAME_MAX - 1);
 		strtok(con, ".");
 
-		/* add relevant controller */
-		cgc = cgroup_add_controller(src_cgroup, con);
+		/* find out whether we have to add the controller or
+		   cgroup already contains it */
+		cgc = cgroup_get_controller(src_cgroup, con);
 		if (!cgc) {
-			fprintf(stderr, "controller %s can't be add\n",
-					con);
-			goto scgroup_err;
+			/* add relevant controller */
+			cgc = cgroup_add_controller(src_cgroup, con);
+			if (!cgc) {
+				fprintf(stderr, "controller %s can't be add\n",
+						con);
+				goto scgroup_err;
+			}
 		}
 
 		/* add name-value pair to this controller */
@@ -595,3 +674,10 @@ char *cgroup_get_value_name(struct cgroup_controller *controller, int index)
 		return NULL;
 }
 
+char *cgroup_get_cgroup_name(struct cgroup *cgroup)
+{
+	if (!cgroup)
+		return NULL;
+
+	return cgroup->name;
+}
