@@ -16,7 +16,7 @@
 #
 
 # TODO path to config.h have to be set properly
-OPAQUE_HIERARCHY=`grep "OPAQUE_HIERARCHY" /home/elrey/git/20210426-libcg-rel/libcgroup/config.h |\
+OPAQUE_HIERARCHY=`grep "OPAQUE_HIERARCHY" /home/thromatka/git/20220511-libcg-rel/upcg/config.h |\
 	 cut -d" " -f3 | sed  's|\"||g'`
 DEBUG=false;		# for debug messages
 FS_MOUNTED=0;		# 0 for not mounted, 1 for mounted, 2 for multimounted
